#include "FixedOrder.h"

using std::string;
using std::domain_error;


double FixedOrder::pi = M_PI;


void FixedOrder::create() 
{
  as = LHAPDF::alphasPDF(muR);
  lg   = 2*log(muR/mt);
  lgfr = 2*log(muF/muR);
}

// The analytic result for the Fixed Order _depending_ on the options.
double FixedOrder::FOgg(double rho)
{
  double f = 0.0;	
  // --- LO:	
  if      (LO) { f += PXS_LO_gg(rho) ;}
  else { f += 0;}
  // --- NLO:	
  if (NLO) 
    { 
      if (NLOparam)//exact CM 
	{
	  f += PXS_NLO_CM_gg(rho) ;
	}
      else//approximate NDE parameterization (Nason, Dawson, Ellis '88)
	{
	  f += PXS_NLO_NDE_gg(rho) ;
	}
    }
  else { f += 0;}
  // --- NNLO:	
  if (NNLO) 
    { 
      if (NNLOapproxORexact)// Use the result for the threshold approximation to the NNLO result 
	{
	  f += PXS_NNLO_APPROX_gg(rho) ;
	}
      else//Use the exact NNLO result
	{
	  f += PXS_NNLO_EXACT_gg(rho) ;
	}
    }
  else { f += 0;}
  
  return f * 0.38937966*pow(10,9)*pow(as/mt,2);
}


double FixedOrder::FOqqbar(double rho)
{
  double f = 0.0;	
  // --- LO:	
  if      (LO) { f += PXS_LO_qqbar(rho) ;}
  else { f += 0;}
  // --- NLO:	
  if (NLO) 
    { 
      if (NLOparam) 
	{
	  f += PXS_NLO_CM_qqbar(rho) ;
	}
      else
	{
	  f += PXS_NLO_NDE_qqbar(rho) ;
	}
    }
  else { f += 0;}
  // --- NNLO:	
  if (NNLO) 
    { 
      if (NNLOapproxORexact) 
	{
	  f += PXS_NNLO_APPROX_qqbar(rho);
	}
      else
	{
	  f += PXS_NNLO_EXACT_qqbar(rho) ;
	}
    }
  else { f += 0;}
  
  return f * 0.38937966*pow(10,9)*pow(as/mt,2);
}


double FixedOrder::FOgq(double rho)
{
  double f = 0.0;	
  // --- LO:	
  if      (LO) { f += 0;}
  else { f += 0;}
  // --- NLO:	
  if (NLO) 
    { 
      if (NLOparam) 
	{
	  f += PXS_NLO_CM_gq(rho) ;
	}
      else
	{
	  f += PXS_NLO_NDE_gq(rho) ;
	}
    }
  else { f += 0;}
  // --- NNLO:	
  if (NNLO) 
    { 
      if (NNLOapproxORexact) 
	{
	  f += PXS_NNLO_APPROX_gq(rho) ;
	}
      else
	{
	  f += PXS_NNLO_EXACT_gq(rho) ;
	}
    }
  else { f += 0;}
  
  return f * 0.38937966*pow(10,9)*pow(as/mt,2);
}



/*******************************************************************************	
*									       *	
*		  The functions parametrizing the NDE result		       *
*									       *	
*******************************************************************************/	

double FixedOrder::h1(double beta)
{
  double f = -2*gsl_sf_dilog((1 - beta)/2.) 
    + 2*gsl_sf_dilog((1 + beta)/2.) 
    - pow(log((1 - beta)/2.),2) 
    + pow(log((1 + beta)/2.),2);
  return f;
}

double FixedOrder::h2(double beta)
{
  double f = -gsl_sf_dilog((-2*beta)/(1 - beta)) 
    + gsl_sf_dilog((2*beta)/(1 + beta));
  return f;
}

double FixedOrder::f0qq(double rho)
{
  double beta = sqrt(1-rho);
  double f = pi*beta*rho/27.*(2+rho);	
  return f;
}

double FixedOrder::f0qqLeading(double rho)//only the O(beta) term (for testing)
{
  double beta = sqrt(1-rho);
  double f = 0.111111*beta*pi;
  return f;
}

double FixedOrder::f0gg(double rho)
{
  double beta = sqrt(1-rho);
  double f = pi*beta*(rho/192.)
    *((pow(rho,2) + 16*rho+16)
      *log((1+beta)/(1-beta))/beta-28-31*rho);	
  return f;
}

double FixedOrder::f0ggLeading(double rho)//only the O(beta) term (for testing)
{
  double beta = sqrt(1-rho);
  double f = 0.0364583*beta*pi;	
  return f;
}

double FixedOrder::f1qqbar(double rho)
{
  double beta = sqrt(1-rho);
  double nlf = 5;
  double f = (2*rho*log((1 + beta)/(1 - beta)))/(81.*pi) 
    + f0qq(rho)*(-(-127 + 6*nlf)/(72.*pow(pi,2)) 
		 + (2*log(rho/(4.*pow(beta,2))))/(3.*pow(pi,2)));
  return f;
}

double FixedOrder::f1ggbar(double rho)
{
  double beta = sqrt(1-rho);	
  double f = (-181*beta)/(1440.*pi) 
    + (26*beta*rho)/(45.*pi) - (2483*beta*pow(rho,2))/(1920.*pi) 
    + (-rho/(8.*pi) + pow(rho,2)/(16.*pi) - pow(rho,3)/(256.*pi))*h1(beta) 
    + (rho/(8.*pi) + pow(rho,2)/(8.*pi) + pow(rho,3)/(128.*pi))*h2(beta) 
    + ((-3*rho)/(8.*pi) + (33*pow(rho,2))/(128.*pi) 
       + (59*pow(rho,3))/(768.*pi))*log((1 + beta)/(1 - beta)) 
    + (3*f0gg(rho)*log(rho/(4.*pow(beta,2))))/(2.*pow(pi,2));
  return f;
}

double FixedOrder::f1gqbar(double rho)
{
  double beta = sqrt(1-rho);	
  double f = (-181*beta)/(6480.*pi) + (289*beta*rho)/(2160.*pi) 
    - (1319*beta*pow(rho,2))/(25920.*pi) 
    + (-rho/(72.*pi) + pow(rho,2)/(144.*pi))*h1(beta) 
    + ((-17*rho)/(432.*pi) + pow(rho,2)/(128.*pi) 
       + (7*pow(rho,3))/(1728.*pi))*log((1 + beta)/(1 - beta));
  return f;
}

double FixedOrder::f1qq(double rho)
{
  double beta = sqrt(1-rho);
  double nlf = 5;
  double a0 = 0.180899,
    a1 = 0.101949,
    a2 = -0.234371,
    a3 = -0.0109950,
    a4 = -0.0185575,
    a5 = 0.00907527,
    a6 = 0.0160367,
    a7 = 0.00786727;
  
  double f = (a0*beta + a2*pow(beta,3) + a4*pow(beta,5) - pi/432.)*rho 
    + (a1*pow(beta,3) + a3*pow(beta,5) + a5*pow(beta,7) 
       - (41*beta)/(108.*pi))*rho*log(8*pow(beta,2)) 
    + (2*beta*rho*pow(log(8*pow(beta,2)),2))/(27.*pi) 
    + f0qq(rho)*((-5*(-4 + nlf))/(36.*pow(pi,2)) 
		 + ((-4 + nlf)*log(4/rho))/(12.*pow(pi,2))) 
    + a6*beta*rho*log(rho) + a7*beta*rho*pow(log(rho),2);
  return f;
}

double FixedOrder::f1gg(double rho)
{
  double beta = sqrt(1-rho);
  double nlf = 5;	
  double a0 = 0.108068,
    a1 = -0.114997,
    a2 = 0.0428630,
    a3 = 0.131429,
    a4 = 0.0438768,
    a5 = -0.0760996,
    a6 = -0.165878,
    a7 = -0.158246;
  
  double f = a0*beta + a2*pow(beta,3) + (11*pi)/9216. 
    - (beta*(-4 + nlf)*pow(rho,2))/(512.*pi)
    + (a1*pow(beta,3) + a3*pow(beta,5) - (61*beta)/(256.*pi))*log(8*pow(beta,2)) 
    + (7*beta*pow(log(8*pow(beta,2)),2))/(128.*pi) 
    + ((-4 + nlf)*pow(rho,2)*log((1 + beta)/(1 - beta)))/(1024.*pi) 
    + (a6*beta*rho + a4*beta*pow(rho,2))*log(rho) 
    + (a7*beta*rho + a5*beta*pow(rho,2))*pow(log(rho),2);
  return f;
}

double FixedOrder::f1gq(double rho)
{
  double beta = sqrt(1-rho);
  double a0 = 0.0110549,
    a1 = -0.426773,
    a2 = -0.00103876,
    a3 = 0.450626,
    a4 = -0.227229,
    a5 = 0.0472502,
    a6 = -0.197611,
    a7 = -0.0529130;
  
  double f = a1*pow(beta,3) + a3*pow(beta,5) 
    + (a0*pow(beta,3) + a2*pow(beta,5))*log(beta) 
    + (a6*beta*rho + a4*beta*pow(rho,2))*log(rho) 
    + (a7*beta*rho + a5*beta*pow(rho,2))*pow(log(rho),2);
  return f;
}


//====================================== LO Partonic X-section:
double FixedOrder::PXS_LO_qqbar(double rho)
{
  double f = f0qq(rho);
  return f;
}

double FixedOrder::PXS_LO_gg(double rho)
{
  double f = f0gg(rho);
  return f;
}

//====================================== NLO Partonic X-section:
double FixedOrder::PXS_NLO_NDE_gg(double rho)
{
  double f = f0gg(rho)*(-1.2201878970378643*lgfr*as) 
    + 4*pi*as*(f1gg(rho)+f1ggbar(rho)*(lg+lgfr));
  return f;
}

double FixedOrder::PXS_NLO_NDE_qqbar(double rho)
{
  double f = f0qq(rho)*(-1.2201878970378643*lgfr*as)
    +4*pi*as*(f1qq(rho)+f1qqbar(rho)*(lg+lgfr));
  return f;
}

double FixedOrder::PXS_NLO_NDE_gq(double rho)
{
  double f = 4*pi*as*(f1gq(rho)+f1gqbar(rho)*(lg+lgfr));
  return f;
}

//====================================== NNLO Partonic X-section:
double FixedOrder::PXS_NNLO_APPROX_gg(double rho)
{
  double tag;
  if (NNLOonORoff) { tag = 1; }// ON
  else { tag = 0; }// OFF
  
  double beta = sqrt(1-rho);
  double lbe = log(beta);
  double a0 = 4608.0,
    a1 = -1894.91,
    a2 = -912.349,
    a3 = 496.3,	
    a4 = 2456.74,
    a5 = 321.137,
    a6 = 68.5471,
    a7 = -8.62261;
  
  double f = a6/pow(beta,2) + (a7 + a5*lbe + a3*pow(lbe,2))/beta 
    + lbe*(a4 + a2*lbe + a1*pow(lbe,2) + a0*pow(lbe,3));
  
  f += Cbargg + 368.0354152962718*lbe*lg + 2606.7683107762314*pow(lbe,2)*lg 
    - 4608.*pow(lbe,3)*lg - 890.988895989886*lbe*pow(lg,2) + 1152.*pow(lbe,2)*pow(lg,2) 
    + 586.9142055529046*lbe*lgfr + 398.7683107762314*pow(lbe,2)*lgfr 
    - 4608.*pow(lbe,3)*lgfr - 677.9777919797721*lbe*lg*lgfr + 2304.*pow(lbe,2)*lg*lgfr 
    + 213.01110401011397*lbe*pow(lgfr,2) + 1152.*pow(lbe,2)*pow(lgfr,2) 
    + (39.63507799167631*lg - 248.15005351310384*lbe*lg 
       - 79.27015598335262*lgfr - 248.15005351310384*lbe*lgfr)/beta 
    + (-170.99993467225795*lg - 104.29134695624361*pow(lg,2) - 842.9932258973022*lgfr 
       - 547.3482065743074*lg*lgfr - 266.7235262847305*pow(lgfr,2))*tag;
  
  return pow((as/(4.0*pi)),2) * f0gg(rho) * f;
}


double FixedOrder::PXS_NNLO_APPROX_qqbar(double rho)
{
  double tag;
  if (NNLOonORoff) { tag = 1; }// ON
  else { tag = 0; }// OFF
  
  double beta = sqrt(1-rho);
  double lbe = log(beta);
  double a0 = 910.222,
    a1 = -1315.53,
    a2 = 592.292,
    a3 = -140.368,
    a4 = 528.557,
    a5 = 32.106,
    a6 = 3.60774,
    a7 = 3.95105;
  
  double f = a6/pow(beta,2) + a4*lbe + a2*pow(lbe,2) 
    + a1*pow(lbe,3) + a0*pow(lbe,4) + (a7 + a5*lbe + a3*pow(lbe,2))/beta;
  
  f += Cbarqq - 476.01201207116173*lbe*lg + 1358.9947986013012*pow(lbe,2)*lg 
    - 910.2222222222222*pow(lbe,3)*lg - 377.8743498251627*lbe*pow(lg,2) 
    + 227.55555555555554*pow(lbe,2)*pow(lg,2) - 1.970644972971968*lbe*lgfr 
    + 377.6614652679682*pow(lbe,2)*lgfr - 910.2222222222222*pow(lbe,3)*lgfr 
    - 265.0820329836587*lbe*lg*lgfr + 455.1111111111111*pow(lbe,2)*lg*lgfr 
    + 112.792316841504*lbe*pow(lgfr,2) + 227.55555555555554*pow(lbe,2)*pow(lgfr,2) 
    + (-49.34802200544679*lg + 70.18385351885766*lbe*lg + 26.31894506957162*lgfr 
       + 70.18385351885766*lbe*lgfr)/beta 
    + (163.52412710677527*lg + 96.34562657017136*pow(lg,2) - 248.02668254465607*lgfr 
       - 126.53786359824426*lg*lgfr - 46.55015683508219*pow(lgfr,2))*tag;
  
  return pow((as/(4.0*pi)),2) * f0qq(rho) * f;
}

double FixedOrder::PXS_NNLO_APPROX_gq(double rho)
{
  return 0;
}


//--------------Presently unknown functions:

double FixedOrder::PXS_NLO_CM_gg(double rho)
{
  return 0;
}

double FixedOrder::PXS_NLO_CM_qqbar(double rho)
{
  return 0;
}

double FixedOrder::PXS_NLO_CM_gq(double rho)
{
  return 0;
}

double FixedOrder::PXS_NNLO_EXACT_gg(double rho)
{
  return 0;
}

double FixedOrder::PXS_NNLO_EXACT_qqbar(double rho)
{
  return 0;
}

double FixedOrder::PXS_NNLO_EXACT_gq(double rho)
{
  return 0;
}
