/*******************************************************************************	
*									       *	
*	  A class that evaluates the fixed order results through NNLO.	       *
*     It wraps a large number of functions in a simple interface.	       *
*	  Its initialization involves no calculations.			       *
*									       *	
*******************************************************************************/


#ifndef guard_class_FixedOrder_h
#define guard_class_FixedOrder_h

#include <math.h>
#include <vector>
#include <stdexcept>
#include <cstddef>
#include "LHAPDF/LHAPDF.h"
#include <gsl/gsl_sf_dilog.h>


class FixedOrder {
 public:
  
 FixedOrder(double _mt, 
	    double _muF, 
	    double _muR, 
	    bool _LO, 
	    bool _NLO, 
	    bool _NNLO, 
	    bool _NLOparam, 
	    bool _NNLOapproxORexact, 
	    bool _NNLOonORoff, 
	    double _Cbarqq, 
	    double _Cbargg) :
  mt(_mt),
    muF(_muF),
    muR(_muR), 
    LO(_LO),
    NLO(_NLO),
    NNLO(_NNLO),
    NLOparam(_NLOparam),
    NNLOapproxORexact(_NNLOapproxORexact),
    NNLOonORoff(_NNLOonORoff),
    Cbarqq(_Cbarqq),
    Cbargg(_Cbargg)
    { 
      create(); 
    }
  
  double FOgg(double rho);
  double FOqqbar(double rho);
  double FOgq(double rho);
  
 private:
  static double pi;
  double as, mt, muF, muR, lg, lgfr;
  bool LO, NLO, NNLO;
  bool NLOparam, NNLOapproxORexact, NNLOonORoff;
  double Cbarqq, Cbargg;	
  
  void create();	
  
  double PXS_LO_gg(double rho);
  double PXS_NLO_CM_gg(double rho);
  double PXS_NLO_NDE_gg(double rho);
  double PXS_NNLO_APPROX_gg(double rho);
  double PXS_NNLO_EXACT_gg(double rho);
  
  double PXS_LO_qqbar(double rho);
  double PXS_NLO_CM_qqbar(double rho);
  double PXS_NLO_NDE_qqbar(double rho);
  double PXS_NNLO_APPROX_qqbar(double rho);
  double PXS_NNLO_EXACT_qqbar(double rho);
  
  double PXS_NLO_CM_gq(double rho);
  double PXS_NLO_NDE_gq(double rho);
  double PXS_NNLO_APPROX_gq(double rho);
  double PXS_NNLO_EXACT_gq(double rho);
  
  // NDE parametrization:
  double f0gg(double rho);
  double f0qq(double rho);
  double f0ggLeading(double rho);//the O(beta) term only
  double f0qqLeading(double rho);//the O(beta) term only
  double h1(double beta);
  double h2(double beta);
  double f1qqbar(double rho);
  double f1ggbar(double rho);
  double f1gqbar(double rho);
  double f1qq(double rho);
  double f1gg(double rho);
  double f1gq(double rho);
};

#endif
