/*******************************************************************************	
*									       *	
*  - Functions that convolute the perturbative components with the fluxes      *
*  - Two functions that calculate PDF uncertainties (NNPDF and asymmetric)     * 
*									       *	
*******************************************************************************/

#ifndef guard_Utilities_h
#define guard_Utilities_h

#include <stdio.h>
#include <gsl/gsl_statistics.h>

#include "PartonicFlux.h"
#include "SubtrFlux.h"
#include "FixedOrder.h"
#include "Resummation.h"

struct FOcomponents 
{
  PartonicFlux Flux; 
  FixedOrder FO; 
  double rho; 
};		

struct REScomponents 
{
  PartonicFlux Flux; 
  SubtrFlux SFlux; 
  Resummation Res; 
  double rho; 
};		

double ComputeFinalResultFO(PartonicFlux Flux, 
			    FixedOrder FO, 
			    double rho, 
			    int precision);

double ComputeFinalResultRES(PartonicFlux Flux, 
			     SubtrFlux SFlux, 
			     FixedOrder FO, 
			     Resummation Res, 
			     double rho, 
			     int precision);

double f_FO (double x, void * par);
double f_RES (double x, void * par);

double ConvolutionFO(PartonicFlux Flux, 
		     FixedOrder FO, 
		     double rho, 
		     int precision);

double ConvolutionRES(PartonicFlux Flux, 
		      SubtrFlux SFlux, 
		      Resummation Res, 
		      double rho, 
		      int precision);


// If Fact. and Ren. scales pass the scale variation criterion.
bool ScalesPass(std::string RestrictedScaleVariation, 
		double Fact, 
		double Ren);

// Converts OrderRES and OrderFO to integers:
int IntegerForm(std::string );


//-------------------- Calculation of PDF uncerainty


void deltaPDFasymmetric(const std::vector<double>& result, 
			double central, 
			double& smax, 
			double& smin);

void deltaPDFnnpdf(const std::vector<double>& result, 
		   double& smax, 
		   double& smin);

#endif
